import GL = WebGLRenderingContextStrict;
import GL2 = WebGL2RenderingContextStrict;

export class ShaderError extends Error{
    constructor(
        readonly info:string
        ){
            super("Could not compile shader!");
    }
}

export class ProgramError extends Error{
    constructor(
        readonly info:string
        ){
            super("Could not link program");
    }
}

/***
 * compile shader
 */
export function compileShader(gl:GL2, source:string, type: GL.ShaderType): WebGLShader{
    const shader = gl.createShader(type);
    gl.shaderSource(shader,source);
    gl.compileShader(shader);
    const status = gl.getShaderParameter(shader,gl.COMPILE_STATUS);
    if(!status){
        const info = gl.getShaderInfoLog(shader);
        throw new ShaderError(info);
    }
    return shader;
}

export function createProgram(gl:GL2, vertex:WebGLShader,frag:WebGLShader){
    const program = gl.createProgram();
    gl.attachShader(program,vertex);
    gl.attachShader(program,frag);

    gl.linkProgram(program);

    const status = gl.getProgramParameter(program,gl.LINK_STATUS);
    if(!status){
        const info = gl.getProgramInfoLog(program);
        throw new ProgramError(info);
    }
    return program;
}
export function createProgramFromSource(gl:GL2, vert:string,frag:string){
    const vert_shader = compileShader(gl,vert,gl.VERTEX_SHADER);
    const frag_shader = compileShader(gl,frag,gl.FRAGMENT_SHADER);
    return createProgram(gl,vert_shader,frag_shader);
}